/*
* file CcResource.java
*
* Licensed Materials - Property of IBM
* Restricted Materials of IBM
* 
* com.ibm.rational.wvcm.stp.cc.CcResource
*
* (C) Copyright IBM Corporation 2004, 2014.  All Rights Reserved. 
* Note to U.S. Government Users Restricted Rights:  Use, duplication or  
* disclosure restricted by GSA ADP  Schedule Contract with IBM Corp. 
*/


package com.ibm.rational.wvcm.stp.cc;

import javax.wvcm.Feedback;
import javax.wvcm.PropertyRequestItem;
import javax.wvcm.Resource;
import javax.wvcm.WvcmException;

import com.ibm.rational.wvcm.stp.StpResource;

/**
 * <p>A proxy for a resource in a ClearCase VOB or view.
 * </p>
 */
public interface CcResource extends StpResource {

    /**
     * Construct a new proxy for this resource. 
     * This new proxy will be of the most correct, most derived class for this
     * resource, based on currently available information in the resource's
     * location, on the local client machine, and/or on the ClearCase server.
     * In addition, this new proxy will have its identity
     * correctly set so that its equals() and hashMap() methods will work
     * correctly.
     * <p>Unresolved resource proxies should not be used in situations where
     * equality matters, especially in maps, lists, sets, and other collection
     * classes.
     * </p>
     * @return a new proxy of the correct, most specific resource type
     */
    public CcResource doResolve() throws WvcmException;

    /**
     * Has this proxy already been resolved?
     * @return true if this proxy has been resolved; else false
     * @see #doResolve()
     */
    public boolean isResolved() throws WvcmException;
    
    /**
     * Get the ClearCase provider associated with this resource proxy.
     * This is normally the network provider for the CCRC WAN server on which
     * this remote resource resides, or a local provider if this is a local
     * ClearCase resource.
     */
    CcProvider ccProvider();

    /**
     * Checks if this proxy contains valid values for the provided properties, if
     * and only if, they are supported.  Properties which are unsupported by the 
     * server are ignored.
     * 
     * @param wantedProp A PropertyRequestItem, possibly with multiple nested
     *            properties, to check the value for. Cannot be
     *            <code>null</code>.
     * 
     * @return <code>true</code> if there is an entry in this proxy for all
     *         server supported properties of the PropertyRequestItem, 
     *         and those entries contain a valid value; <code>false</code> otherwise.
     */
    public boolean hasSupportedProperties(PropertyRequestItem wantedProp);
    
    /**
     * Persists property changes to this CcResource.
     * An exception is thrown if any of properties cannot be written.
     * If an exception is thrown, the properties that were set will not be written
     * can be determined from {@link #updatedPropertyNameList}
     * on this CcResource.
     * Note that since doWriteProperties returns a new proxy whose properties are
     * specified in the feedback argument, doWriteProperties is semantically identical
     * to {@link #doReadProperties}.
     * Properties are computed relative to the given context. For example, when
     * working with automatic views, the
     * {@link com.ibm.rational.wvcm.stp.cc.CcVobTag#IS_MOUNTED}
     * property of a CcVobTag resource can only be modified in the
     * context of a CcView. Hence, you would use this form of doWriteProperties
     * to specify the CcView context that should be used.
     * 
     * @param context An optional proxy (often CcView) providing context for the
     *            writing of certain properties. May be <b>null</b>.
     * @param feedback Specifies optional feedback to the caller.
     * @return a new proxy for this resource, whose properties are specified by feedback.
     */
    public Resource doWriteProperties(Resource context, 
                                      Feedback feedback) throws WvcmException;

    /**
     * Reads the CcResource's properties if they are available locally on the client machine.
     * Any modified properties in this CcResource are first written to the resource before 
     * the properties are read from the resource. Note that this makes readProperties 
     * semantically identical to {@link #writeProperties}.
     * @param feedback the properties to be available in the returned proxy,
     *  and any other feedback desired, such as progress indications.
     * @return a {@link Resource} containing the wanted properties
     * that are available locally on the client machine
     * without communicating with the server.
     * Note that there is an exception to this for automatic views, where a request to
     * a connected automatic view may result in server communication to update local 
     * cached data. 
     * 
     * @see Resource#doReadProperties(Feedback) doReadProperties.
     * @throws WvcmException
     * <p>
     * {@link javax.wvcm.WvcmException.ReasonCode#NOT_FOUND}:
     * If the resource is not be known locally on the client machine.
     */
    public Resource readProperties(Feedback feedback) throws WvcmException;
    
   /**
     * Request the properties specified in <code>feedback</code> from the
     * client resource represented by this proxy. A new proxy will be
     * constructed whose property map contains just the requested properties.
     * Any modified properties in this CcResource are first written to the resource before 
     * the properties are read from the resource. Note that this makes readProperties 
     * semantically identical to {@link #writeProperties}.
     * <p>
     * Properties are computed within the context of a given resource context. 
     * For example, the {@link com.ibm.rational.wvcm.stp.cc.CcVobTag#IS_MOUNTED}
     * property of a CcVobTag for automatic views can be provided in the
     * context of a CcView. Hence, you would use this form of readProperties
     * to obtain the list of vob tags mounted for the specified view context.
     * 
     * @param context An optional proxy (often CcView) providing context for the
     *            generation of certain properties in the returned report. 
     *            May be <b>null</b>.
     * @param feedback Provides optional feedback to the caller for a list of 
     *            properties desired from the client. If the parameter is null or
     *            empty, a request is still made, and the returned proxy will have
     *            only the properties always requested from the client.
     * @return A proxy containing the properties retrieved from the client (and
     *         error messages for those that could not be retrieved). The class
     *         of the returned proxy is the one most suited to the type of
     *         resource addressed.
     * 
     * @throws WvcmException if failures occur
     * 
     * @see CcResource#readProperties(Feedback)
     */
    public Resource readProperties(Resource context, Feedback feedback) throws WvcmException;

    /**
     * Resolve this ClearCase resource, but do not consult the ClearCase server.
     * Unlike {@link CcResource#doResolve()}, use only information currently
     * available information on the local client machine.  
     * @see CcResource#doResolve()
     * @return a new ClearCase resource proxy of the correct, most specific resource type
     * @throws WvcmException with NOT_FOUND reason code if this resource does not
     *         exist on the local machine.
     */
    public Resource resolve() throws WvcmException;
    
    /**
     * Persists property changes to this CcResource locally if they can be persisted locally.
     * An exception is thrown if any of properties cannot be written.
     * If an exception is thrown, the properties that were set will not be written
     * can be determined from {@link #updatedPropertyNameList}
     * on this CcResource.
     * Note that since writeProperties returns a new proxy whose properties are
     * specified in the feedback argument, writeProperties is semantically identical
     * to {@link #readProperties}.
     * @param feedback Specifies optional feedback to the caller.
     * @return a new proxy for this resource, whose properties are specified by feedback.
     * @see Resource#doWriteProperties(Feedback)
     */
    public Resource writeProperties(Feedback feedback) throws WvcmException;
    
    /**
     * Persists property changes to this CcResource locally if they can be persisted locally.
     * An exception is thrown if any of properties cannot be written.
     * If an exception is thrown, the properties that were set will not be written
     * can be determined from {@link #updatedPropertyNameList}
     * on this CcResource.
     * Note that since writeProperties returns a new proxy whose properties are
     * specified in the feedback argument, writeProperties is semantically identical
     * to {@link #readProperties}.
     * Properties are computed within the context of a given resource context. For example,
     * when working with automatic views, the {@link com.ibm.rational.wvcm.stp.cc.CcVobTag#IS_MOUNTED}
     * property of a CcVobTag resource can only be modified in the
     * context of a CcView. Hence, you would use this form of writeProperties
     * to specify the view context that should be used.
     * 
     * @param context An optional proxy (often CcView) providing context for the
     *            writing of certain properties. May be <b>null</b>.
     * @param feedback Specifies optional feedback to the caller.
     * @return a new proxy for this resource, whose properties are specified by feedback.
     */
    public Resource writeProperties(Resource context, Feedback feedback) throws WvcmException;
}
